# msmnt.sh - M$ Mounter Emulator Functions.
#
# Copyright 2007 - J. S. Gilstrap   All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#===============================================================================
#
# Place in /etc/profile.d or in /etc and source from /etc/bashrc

# Check for a user configuration before checking
# for a global configuration and source it.
if [ -f ~/.msmntrc ]
then
	. ~/.msmntrc

elif [ -f /etc/msmntrc ]
then
	 . /etc/msmntrc
else
	 echo "Can't Find a Configuration File for the M\$ Mounter BugFeature."
fi

# Main M$ Mounter Function.
msmnt()
	{
	# If called with no $1 option, quit and do nothing.
	if [ -z "$1" ]
	then
		echo "'msmnt' is a functuon that emulates the Micro\$oft mounting characteristics for"
		echo "removeable devices at the DOS prompt and must be called by a parent function where"
		echo "'\$1' is a mount point for the asssociated device located in the root filesystem."						 
		return	
	fi

	mkstk

	# Initialize some variables.
		   MP=$1
	   isMnt=`mount | awk '$3 ~ /'\\\/${MP}'/ {print $0}'`
	   isCWD=`pwd | grep "^/${MP}"`
	isOldCWD=`echo $OLDPWD | grep "^/${MP}"`

	# Mount if unmounted with no unmount option
	# specified and cd to the mount point.
	if [ -z "$isMnt" -a -z "$2" ]
	then
		SWD=$PWD
		cdhow
		mount /${MP} > /dev/null 2>&1 ; Err=$?

		if [ $Err -eq 0 ]
		then
			if [ "$Umsg" = yes ]
			then
				echo "Remember to unmount with 'u' or 'q' as an option before physically removing"
				echo "media and option 'e' will unmount and physically eject media, i.e. CD-ROM."
			fi

			builtin cd /${MP}
			mkstk
			pwd >> /tmp/.dir_stk.$$
			chmod 600 /tmp/.dir_stk.$$
		else
			[ $SWD != $PWD ] && cd $SWD
			echo "Not Mountable."
		fi

	# If not mounted eject if possible on 'e'.
	elif [ -z "$isMnt" -a "$2" = e ]
	then
		eject /${MP} > /dev/null 2>&1					  

	# Unmount if mounted when the unmount option 'u', 'q', or 'e' is 
	# specified and remove the associated directory from the stack.
	# If 'e' is specified then attempt to eject.
	elif [ -n "$isMnt" ] && [ "$2" = u -o "$2" = q -o "$2" = e ]
	then
		cdhow
		umount /${MP} > /dev/null 2>&1 ; Err=$?

		if [ $Err -eq 0 ]
		then
			delstk
			[ "$2" = e ] && eject /${MP} > /dev/null 2>&1
		else
			echo "Unable to UnMount."
		fi

	# If mounted with no unmount option specified then
	# cd to the associated directory in the stack if
	# available. If not cd to its mount point and place
	# an entry in the stack.
	elif [ -n "$isMnt" -a -z "$2" ]
	then
		 cdstk
	fi

	# Empty variables when done.
	unset MP isMnt isCWD isOldCWD SWD Err
	}

# CD to directory in stack.
# If the mount point exists in the stack then cd into it.
cdstk()
	{
	mkstk
	Dir=`grep "^/${MP}" /tmp/.dir_stk.$$`

	if [ -n "$Dir" ]
	then
		builtin cd "$Dir"
	else
		builtin cd /${MP}
		pwd >> /tmp/.dir_stk.$$
		chmod 600 /tmp/.dir_stk.$$
	fi
	
	# Empty variable when done.
	unset Dir
	}

# Determine how to cd out of a mount point before (un)mounting.
cdhow()
	{
	if [ -n "$isCWD" ]
	then
		if [ -n "$OLDPWD" -a -z "$isOldCWD" ]
		then
			builtin cd - > /dev/null
		else
			builtin cd ~
		fi
	fi
	}

# Make the Directory Stack File. 
mkstk()
	{
	if [ ! -f /tmp/.dir_stk.$$ ]
	then
		if echo $MP | grep -q "home"
		then
			pwd | grep "^/home" >> /tmp/.dir_stk.$$
		else
			echo $HOME >> /tmp/.dir_stk.$$
		fi

		chmod 600 /tmp/.dir_stk.$$
	fi
	}

# Remove a directory path from the stack.
#
# There is probably an easier way to remove a directory listing
# from the stack instead of moving it to .old, create a new one
# and deleting the .old one. Maybe using sed or something else.
delstk()
	{
	mkstk
	mv    /tmp/.dir_stk.$$     /tmp/.dir_stk.$$.old
	cat   /tmp/.dir_stk.$$.old | grep -v "^/${MP}" > /tmp/.dir_stk.$$
	rm -f /tmp/.dir_stk.$$.old > /dev/null 2>&1
	chmod 600 /tmp/.dir_stk.$$
	}

# Monitor cd commands and update the stack if necessary to
# the directory path associated with the mount point.
cd()
	{
	builtin cd "$@"
	MP=`pwd | cut -d / -f 2`
	mkstk

	if [ -n "$MP" ] && grep -q "^/${MP}" /tmp/.dir_stk.$$
	then
		delstk
		pwd >> /tmp/.dir_stk.$$
		chmod 600 /tmp/.dir_stk.$$
	fi

	# Empty variable when done.
	unset MP
	}	

# Print Mount Aliases
malias()
	{
	L=`set | grep ": ()" | grep -v set | awk '{print $1}'`
	echo $L
	unset L
	}

